@@ -1,11 +1,25 @@
 #ifndef BSON_CONFIG_H
 #define BSON_CONFIG_H
 
-/*
- * Define to 1234 for Little Endian, 4321 for Big Endian.
- */
+/* stuff for solaris */
+#if (defined(__sun) && defined(__SVR4))
+#define BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES 1
+#define BSON_HAVE_ATOMIC_32_ADD_AND_FETCH 0
+#define BSON_HAVE_ATOMIC_32_ADD_AND_FETCH 0
+/* sparc is big endian */
+#include <sys/byteorder.h>
+#ifdef _BIG_ENDIAN
+#define BSON_BYTE_ORDER 4321
+#else
 #define BSON_BYTE_ORDER 1234
-
+#endif
+#else
+/* for everyone else */
+#define BSON_BYTE_ORDER 1234
+#define BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES 0
+#define BSON_HAVE_ATOMIC_32_ADD_AND_FETCH 1
+#define BSON_HAVE_ATOMIC_32_ADD_AND_FETCH 1
+#endif
 
 /*
  * Define to 1 if you have stdbool.h
@@ -19,8 +33,11 @@
 /*
  * Define to 1 for POSIX-like systems, 2 for Windows.
  */
+#ifdef _WIN32
+#define BSON_OS 2
+#else
 #define BSON_OS 1
-
+#endif
 
 /*
  * Define to 1 if we have access to GCC 32-bit atomic builtins.
@@ -28,7 +45,6 @@
  * dependent. For example, some PPC or ARM systems may not have it even
  * if it is a recent GCC version.
  */
-#define BSON_HAVE_ATOMIC_32_ADD_AND_FETCH 1
 #if BSON_HAVE_ATOMIC_32_ADD_AND_FETCH != 1
 # undef BSON_HAVE_ATOMIC_32_ADD_AND_FETCH
 #endif
@@ -36,7 +52,6 @@
 /*
  * Similarly, define to 1 if we have access to GCC 64-bit atomic builtins.
  */
-#define BSON_HAVE_ATOMIC_64_ADD_AND_FETCH 1
 #if BSON_HAVE_ATOMIC_64_ADD_AND_FETCH != 1
 # undef BSON_HAVE_ATOMIC_64_ADD_AND_FETCH
 #endif
@@ -46,7 +61,6 @@
  * Define to 1 if your system requires {} around PTHREAD_ONCE_INIT.
  * This is typically just Solaris 8-10.
  */
-#define BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES 0
 #if BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES != 1
 # undef BSON_PTHREAD_ONCE_INIT_NEEDS_BRACES
 #endif
