@@ -16,7 +16,7 @@
 
 
 #include "mongoc-counters-private.h"
-#include "mongoc-client-pool.h"
+#include "mongoc-client-pool-private.h"
 #include "mongoc-queue-private.h"
 #include "mongoc-thread-private.h"
 #include "mongoc-cluster-private.h"
